
clear all; close all; clc; % clean slate

% try running these commands one by one to see what is happening

% Solving a couple of nonlinear equations in x1 and x2
x0 = [0; 0]; % some initial guess or starting point for the answer
xresult = fsolve(@someequations,x0);
% fsolve exits saying "Optimization terminated: first-order optimality is
% less than options.TolFun". That is supposed to be a good thing. We can
% check that fsolve was successful by substituting xresult back in the
% equations, to make sure we get very close to zero.
somequations(xresult)

% Because we have only 2 eqns in 2 unkns, we can use 'ezplot' to get an    
% intuitive feel for these equations
figure(1); hold on;
ezplot('x-2*y^2-1'); % x1 replaced by x and x2 replaced by y
ezplot('y-0.5*x^2+2'); 
% the points of intersection of the two curves are the solutions that we
% need. 

% Sometimes fsolve will not converge, because the starting point or the
% initial guess x0 was too far away. Then you can verify that fsolve has
% not converged properly by substituting xresult back in the equations to
% see that they do not evaluate to zero.
% In this case, you just use a different starting point, until you get
% fsolve to converge. 

% Sometimes you may not be able to find a good starting
% point. This can mean one of two things: 1) you have not found a good
% starting point. 2) the equations do not have a solution. It is generally
% impossible to distinguish between these two possibilities simply from a
% failure of fsolve.
