% Two link manipulator, OAB. O = shoulder, A = elbow, B = endpoint = hand.
% This program solves the inverse kinematics problem, using fsolve. 
% That is, find the joint angles that are required to reach a particular end point

clear all; close all; clc;

% Lengths of the two links
param.L1 = 1; param.L2 = 1;

% given theta, finding the end point
% theta1 = pi/6; theta2 = pi/3;
% endpointposition = twolinkforward([theta1; theta2], param)

% now, we find theta such that the end point is at a desired position
endpoint_desired = [0.5; 1]; 
theta_initialguess = [-0.1; 25];
thetaresult = ...
    fsolve(@twolink_endpointmismatch,theta_initialguess,[],endpoint_desired,param);


% Let's check that we have got the correct theta
twolink_endpointmismatch(thetaresult,endpoint_desired,param) % this should evaluate to close to zero

% Let's check another way by drawing a picture of the link.
figure(1); hold on; 
plot(0,0,'.'); % origin of the system 
axis equal; axis([-2 2 -2 2]);
xA = param.L1*cos(thetaresult(1)); yA = param.L1*sin(thetaresult(1));
xB = xA+param.L2*cos(thetaresult(1)+thetaresult(2));
yB = yA+param.L2*sin(thetaresult(1)+thetaresult(2));
plot([0 xA],[0 yA],'r'); % upper arm
plot([xA xB],[yA yB],'b'); % fore arm
plot(endpoint_desired(1),endpoint_desired(2),'ro'); % end point desired
text(0,-0.1,'Shoulder');
text(endpoint_desired(1),endpoint_desired(2)+0.1,'Desired end-point');