% Two link manipulator
% This program attempts to make the end point B of the two link manipulator
% track some user-defined curve

clear all; close all; clc;
param.L1 = 1; param.L2 = 0.5;


% Plot the reachable workspace, as computed in the first couple of lectures
% The reachable workspace is an annulus from |L1-L2| to L1+L2.
% So we draw a couple of circles here
figure(1); hold on;
philist = linspace(0,2*pi,1000);
plot((param.L1+param.L2)*cos(philist),(param.L1+param.L2)*sin(philist),'b');
plot(abs(param.L1-param.L2)*cos(philist),abs(param.L1-param.L2)*sin(philist),'r');
axis equal; axis([-(param.L1+param.L2) (param.L1+param.L2) ...
    -(param.L1+param.L2) (param.L1+param.L2)]);
xlabel('x'); ylabel('y');


% Pick a few points on the plane and store them to define a curve inside
% the reachable workspace
Numpoints = 10; % can change this in case we want to specify more or less points
figure(1); % makes figure-1 active
xlist_desired = zeros(Numpoints,1); ylist_desired = zeros(Numpoints,1);
for i = 1:Numpoints
    title(['Click on ' num2str(Numpoints-i+1) ' more points']);
    [xtemp,ytemp] = ginput(1); % get a "point" as input by clicking on a figure
    xlist_desired(i) = xtemp; ylist_desired(i) = ytemp;
    plot(xlist_desired(1:i),ylist_desired(1:i),'b.-'); % connect the dots so far
end
title('');



% Now we find the theta's corresponding to each of the points defining the
% curve to be tracked
theta1_desired = zeros(Numpoints,1); theta2_desired = zeros(Numpoints,1);
theta_initialguess = [0; 0];
for i = 1:Numpoints
    endpoint_desired = [xlist_desired(i); ylist_desired(i)];
    thetaresult = ...
        fsolve(@twolink_endpointmismatch,theta_initialguess,[],endpoint_desired,param);
    theta1_desired(i) = thetaresult(1); theta2_desired(i) = thetaresult(2);
    theta_initialguess = thetaresult; % this line is to make likely that
%     fsolve finds the theta-solution close to the previous solution, as
%     opposed to a far-away solution (note inverse kinematics has
%     non-unique solutions). Without this line, the manipulator may "jump
%     around"
end


% Now, we have got the sequence of joint angles that make the end point go
% through the sequence of points. So let's try to do an animation of the
% arm
figure(1);
plot(0,0,'ks'); hold on;
hOA = plot([0 param.L1],[0 0],'k.-'); set(hOA,'Linewidth',1);
hAB = plot([param.L1 param.L1+param.L2],[0 0],'k.-'); set(hAB,'Linewidth',1);
for i = 1:Numpoints
    theta1 = theta1_desired(i); theta2 = theta2_desired(i);
    xA = param.L1*cos(theta1); yA = param.L1*sin(theta1);
    xB = xA+param.L2*cos(theta1+theta2);
    yB = yA+param.L2*sin(theta1+theta2);
    set(hOA,'xdata',[0 xA],'ydata',[0 yA]);
    set(hAB,'xdata',[xA xB],'ydata',[yA yB]);
    pause;
end
